"use client";

import { createIbcRequest, deleteIbc, editIbcRequest } from "@/app/lib/ibc/action";
import { InputWrapper } from "@/components/fields/Fields";
import FormComponent from "@/components/form/Form";
import DataTable from "@/components/tables/DataTable";
import prepareValidationSchema, { FormObject } from "@/utils/validation-schema";
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";
import { Button, Flex, FormControl, FormLabel, Icon, Modal, ModalBody, ModalCloseButton, ModalContent, ModalFooter, ModalHeader, ModalOverlay, Text, useColorModeValue, useDisclosure, useToast } from "@chakra-ui/react";
import { useRouter } from "next/navigation";
import { useEffect, useMemo, useRef, useState } from "react";
import { ClubOwnerInterface } from "../type/ibc";
import { columnsDataBlocks } from "../variables/columnsBlock";

const formFields: FormObject[] = [
  {
    name: "area_name",
    label: "Area Name",
    type: "text",
    defaultValue: '',
    required: true,
  },
  {
    name: "ibc_code",
    label: "Ibc Code",
    type: "number",
    defaultValue: '',
    required: true,
  },
];

interface ClubOwnerTableProps {
  data: ClubOwnerInterface[]; // This is required
}

interface formValues {
  club_admin_share: string;
}

const IbcTable: React.FC<ClubOwnerTableProps> = ({ data }: { data: ClubOwnerTableProps }) => {
  const { isOpen, onOpen, onClose } = useDisclosure();
  const [formValidationSchema, setFormValidationSchema] = useState({});
  const brandColor = useColorModeValue("brand.500", "white");
  const brandStars = useColorModeValue("brand.500", "brand.400");
  const textColor = useColorModeValue("navy.700.700", "white");
  const formDetails = useMemo(() => prepareValidationSchema(formFields), []);
  const [ownerForm, setOwnerForm] = useState({});
  const [currentEditUser, setCurrentEditUser] = useState<ClubOwnerInterface>();
  const { initialFormData, validationSchema } = formDetails;
  const formRef = useRef(null);



  const toast = useToast();
  const router = useRouter();

  useEffect(() => {
    setOwnerForm(initialFormData);
    setFormValidationSchema(validationSchema);
  }, [initialFormData, validationSchema]);

  const deleteUser = async (id: number) => {
    console.log('id', data?.data?.[id]?.id)
    const res = await deleteIbc(data?.data?.[id]?.id);
    if (res) {
      toast({
        title: "200",
        description: res.message,
        status: "success",
        position: "top-right",
        duration: 2000,
        isClosable: true,
      });
      router.refresh();
    } else {
      toast({
        title: res.status,
        description: res.error,
        status: "error",
        position: "top-right",
        duration: 9000,
        isClosable: true,
      });
    }
  };

  const handleAddClick = () => {
    setCurrentEditUser(null);
    onOpen();
  }

  const handleEditClick = async (id: number) => {
    setOwnerForm((prev) => {
      return { ...prev };
    });
    setCurrentEditUser((prev) => {
      return { ...data?.data?.[id] };
    });
    onOpen();
  };

  const handleSubmitClick = () => {
    formRef?.current?.externalHandleSubmit();
  };

  const handleSubmit = async (values: formValues, data: object, isEdit: ClubOwnerInterface) => {
    if (isEdit) {
      const payload = {
        userId: isEdit?.id,
        data: values,
      };
      const res = await editIbcRequest(payload);
      if (res) {
        toast({
          title: "200",
          description: res?.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: '400',
          description: res?.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    } else {
      console.log('values', values)
      const res: any = await createIbcRequest(values);

      if (res) {
        toast({
          title: "200",
          description: res?.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
      } else {
        toast({
          title: '400',
          description: res?.message,
          status: "error",
          position: "top-right",
          duration: 9000,
          isClosable: true,
        });
      }
    }

  };

  const action = (id: number) => (
    <Flex gap={2}>
      <Button
        variant="blank"
        onClick={() => {
          handleEditClick(id);
        }}
        alignItems="center"
        justifyContent="center"
        w="37px"
        h="37px"
        lineHeight="100%"
        borderRadius="10px">
        <Icon as={EditIcon} color={brandColor} w="20px" h="20px" />
      </Button>
      <Button
        variant="blank"
        alignItems="center"
        justifyContent="center"
        w="37px"
        h="37px"
        lineHeight="100%"
        borderRadius="10px"
        onClick={() => {
          deleteUser(id);
        }}>
        <Icon as={DeleteIcon} color={brandColor} w="20px" h="20px" />
      </Button>
    </Flex>
  );

  return (
    <>

      <DataTable
        addButton={<>
          <Button
            variant='solid'
            colorScheme='blue'
            onClick={handleAddClick}
            alignItems="center"
            justifyContent="center"
            lineHeight="100%"
            borderRadius="10px">
            Create
          </Button>
        </>}
        columnsData={columnsDataBlocks}
        tableData={data.data as any[]}
        isSelectable={false}
        showPageSize={false}
        actionSlot={action}
      />

      {isOpen && (
        <Modal onClose={onClose} size="lg" isOpen={isOpen}>
          <ModalOverlay />
          <ModalContent bg={"navy.900"}>
            <ModalHeader>Edit Share</ModalHeader>
            <ModalCloseButton />
            <ModalBody>
              <FormComponent
                ref={formRef}
                validationSchema={formValidationSchema}
                initialValues={ownerForm}
                submitForm={(formValues, formData) => {
                  handleSubmit(formValues, formData, currentEditUser);
                }}>
                <FormControl w="full">
                  <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                    Area Name
                    <Text color={brandStars}>*</Text>
                  </FormLabel>
                  <InputWrapper
                    form={true}
                    fieldName="area_name"
                    defaultValue={currentEditUser?.area_name}
                    id="area_name"
                    variant="auth"
                    w="full"
                    mb="2px"
                    fontSize="sm"
                    ms={{ base: "0px", md: "0px" }}
                    type="text"
                    fontWeight="500"
                  />
                </FormControl>
                <FormControl w="full">
                  <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                    IBC Code
                    <Text color={brandStars}>*</Text>
                  </FormLabel>
                  <InputWrapper
                    form={true}
                    fieldName="ibc_code"
                    defaultValue={currentEditUser?.ibc_code}
                    id="ibc_code"
                    variant="auth"
                    w="full"
                    mb="2px"
                    fontSize="sm"
                    ms={{ base: "0px", md: "0px" }}
                    type="number"
                    minLength={10}
                    fontWeight="500"
                  />
                </FormControl>
              </FormComponent>
            </ModalBody>
            <ModalFooter>
              <Button variant="brand" onClick={handleSubmitClick} type="submit" mr={3}>
                Save
              </Button>
              <Button onClick={onClose}>Cancel</Button>
            </ModalFooter>
          </ModalContent>
        </Modal>
      )}
    </>
  );
};

export default IbcTable;
